/* globals require: true */
/* globals fs: true */
/* globals makeXML: true */
/* globals $: true */
/* exported communicateDocumentOpened */
var communicateInitialized = false;
var communicatePort = 0;
var nodePath = require("path");
var net = require("net");
const { platform } = require("os");
var communicatePropertiesName = "adpoint_nativeapp.properties";
var propertiesFileString = "com.ix.shared.system.LocalDirs.dataDir";
var clientID = "";
var client = null;
var ix_pathToXML = null;
var ix_currentOpCode = "";
const util = require("util");
// const OpenIndesign = require("open-Indesign");
/**
 * Libray to open Indesign from browser by looking for config files downloaded by the browser.
 * @type {{watch: () => void, save: (matspecName:string) => Promise<{success: boolean}>, save_and_export: (matspecName:string) => Promise<{success: boolean, payload: string}>, removeDir: (path:string) => void}} OpenIndesign
 */
// const Animations = require("open-Indesign/another");
const OpenIndesign = require("open-Indesign/openIndesign");

// printLogToDebugTXT()
/**
 * Call this function to print log to file instead of debug panel
 */
function printLogToDebugTXT() {
  var logFile = fs.createWriteStream(__dirname + "/debug.txt", { flags: "w" });
  // Or 'w' to truncate the file every time the process starts.
  var logStdout = process.stdout;

  console.log = function () {
    const date = new Date();
    logFile.write(
      date.toUTCString() +
        ": " +
        util.format.apply(null, arguments) +
        "\n" +
        "\n"
    );
    logStdout.write(
      date.toUTCString() +
        ": " +
        util.format.apply(null, arguments) +
        "\n" +
        "\n"
    );
  };
  console.error = console.log;
}

$(function () {
  console.log("communication start");
  OpenIndesign.watch();
});

function communicateDocumentOpened(path, docType) {
  console.log("communicateDocumentOpened " + path);
  try {
    if (communicateInitialized === false) {
      communicateInitialized = true;
      communicateInitialize(path, docType);
    } else {
      console.log("communicateDocumentOpened initialized already");
    }
  } catch (e) {
    console.log("CommunicateDocumentOpened Exception " + e.message);
  }
}

function communicateInitialize(path, docType) {
  console.log("      communicateInitialize " + path);
  communicatePort = communicateGetPort(path); // n.b. - this sets up path to xml file we read later for port info
  console.log("     ix_pathToXML  is " + ix_pathToXML);
  //if(communicatePort !== 0) {
  communicateSendInitializeToAdPoint(docType);
  //}
}

function communicateGetPort(path) {
  console.log("communicateGetPort " + path);
  var portRet = null;
  do {
    var buf = communicateReadAdPointNativeAppProperties(path);
    if (buf === null) {
      break;
    }
    ix_pathToXML = communicateGetPathToAdPointNativeAppXML(buf);
    if (ix_pathToXML === null) {
      break;
    }
    portRet = communicateGetPortFromXML(ix_pathToXML);
  } while (false);
  return portRet;
}

function communicateReadAdPointNativeAppProperties(path) {
  //console.log("communicateReadAdPointNativeAppProperties " + path);
  var bufRet = null;
  do {
    var propPath = communicateGetPropertiesPath(path);
    if (propPath === null) {
      break;
    }
    bufRet = communicateReadPropertiesFile(propPath);
  } while (false);
  return bufRet;
}

function communicateGetPropertiesPath(path) {
  //console.log("communicateGetPropertiesPath " + path);
  var pathRet = null;
  // the path is two levels up
  var up1 = nodePath.dirname(path);
  //console.log("communicateGetPropertiesPath up1 " + up1);
  var up2 = nodePath.dirname(up1);
  //console.log("communicateGetPropertiesPath up2 " + up2);
  var up3 = nodePath.dirname(up2);
  //console.log("communicateGetPropertiesPath up3 " + up3);
  pathRet = nodePath.join(up3, communicatePropertiesName);
  //console.log("communicateGetPropertiesPath pathRet " + pathRet);
  return pathRet;
}

function communicateReadPropertiesFile(path) {
  //console.log("communicateReadPropertiesFile " + path);
  var buf = null;
  try {
    buf = fs.readFileSync(path, "utf8");
  } catch (e) {
    console.log("communicateReadPropertiesFile exception " + e.message);
  }
  //console.log("communicateReadPropertiesFile read " + buf);
  return buf;
}

function communicateGetPathToAdPointNativeAppXML(buf) {
  //console.log("communicateGetPathToAdPointNativeAppXML " + buf);
  var path = null;
  var arr = buf.split("\r");
  //console.log("communicateGetPathToAdPointNativeAppXML " + arr.length + " lines");

  for (var i = 0; i < arr.length; i++) {
    if (arr[i].indexOf(propertiesFileString) !== -1) {
      path = arr[i].substr(propertiesFileString.length + 1);
      break;
    }
  }

  //console.log("communicateGetPathToAdPointNativeAppXML path " + path);

  return path;
}

function communicateGetPortFromXML(path) {
  console.log("communicateGetPortFromXML path " + path);
  var portRet = null;
  try {
    do {
      var buf = fs.readFileSync(path, "utf8");
      //console.log("communicateGetPortFromXML buf " + buf);
      var xmlBuf = makeXML(buf);
      if (xmlBuf === null) {
        break;
      }
      portRet = xmlBuf.find("adpoint").find("plugin").attr("port");
    } while (false);
  } catch (e) {
    console.log("communicateGetPortFromXML exception " + e.message);
  }
  return portRet;
}

function communicateSendInitializeToAdPoint(docType) {
  if (docType == "PlanPoint" || docType == "AdPoint") {
    var initializeData =
      "<request cid='-1' application='" +
      localStorage.ixApplicationName +
      "' action='initialize'></request>\r\n\r\n";
    console.log("communicateSendInitializeToAdPoint " + initializeData);
    communicateSendIt(initializeData, communicateDidInitialize);
  } else {
    console.log(
      " communicateSendInitializeToAdPoint error unknown comm init " + docType
    );
  }
}

function communicateDidInitialize(data) {
  var retString = data.toString();
  console.log("communicateDidInitialize " + retString);
  var xmlBuf = makeXML(retString);
  if (xmlBuf !== null) {
    clientID = xmlBuf.find("response").attr("cid");
    console.log("communicateDidInitialize client id: " + clientID);
  } else {
    console.log("communicateDidInitialize invalid return cannot make clientID");
  }
}

// TODO add more option types
/**
 * @param {string} path
 * @param {'SaveAndExport' | 'Save' | 'SaveClose' | 'SaveAndExportCustom'} option button option
 */
async function communicateExport(path, option) {
  if (!path) {
    console.log("communicateExport - xml making error");
    return;
  }

  //   TODO insert save

  console.log("option: " + option);
  console.log("path to jobbag: ", path);
  const slash = platform() === "win32" ? "\\" : "/";

  console.log("slash ", slash);

  const splittedPath = path.split(slash);
  const matspecName = splittedPath[splittedPath.length - 2];

  console.log("matspecName:", matspecName);

  if (option === "SaveAndExport" || option === "SaveAndExportCustom") {
    console.log("called save and export");

    const { success, payload } = await OpenIndesign.save_and_export(
      matspecName
    );

    if (success) {
      const cb = () => {
        if (payload !== undefined) {
          console.log("Removed folder: ", payload);
          OpenIndesign.removeDir(payload);
        }
      };

      const tcb = setTimeout(cb, 5000);

      const curry = () => tcb;

      doScript("ix.closeThisDoc", curry);
    }
  } else {
    console.log("called save");
    const { success } = await OpenIndesign.save(matspecName);

    if (success && option === "SaveClose") {
      doScript("ix.closeThisDoc");
    }
  }

  return;

  //var arr = paths.split("$");
  // var xmlPath = path; //arr[0] + arr[1];
  // //var docPath = arr[0] + arr[2];
  // //var changedName = xmlPath.replace("exported", "saved");
  // var buf = fs.readFileSync(xmlPath, "utf8");
  // //var res = buf.replace("<resource", "<request cid ='" + clientID + "' action='save' pdfpath='"+arr[3]+"' path='" + docPath + "'");
  // //var res2 = res.replace("/resource", "/request");
  // buf = buf + "\r\n\r\n\r\n";
  // //var res3 = res2;
  // //var res3 = res2.replace(/"/g, "'");
  // //fs.writeFileSync(changedName, res2, 'utf8');
  // //fs.unlinkSync(xmlPath); // delete _exported xml file
  // //console.log("communicateExport file written to " + changedName);
  // //communicateSendIt(res3, communicateDidExport);

  // communicateSendIt(buf, commListen);
}

//function communicateDidExport(result) {
//	console.log("communicateDidExport result: " + result.toString());
//}

function communicateSendIt(buf, dataRoutine) {
  myDataRoutine = dataRoutine;
  if (client !== null) {
    console.log("already connected sending " + buf);
    client.write(buf);
  } else {
    communicatePort = communicateGetPortFromXML(ix_pathToXML); // always look up port pre-connect

    if (communicatePort == null) {
      writeToExtensionConsole("Unable to find comm port. " + ix_pathToXML);
      return;
    }

    client = new net.Socket();
    client.connect(communicatePort, "127.0.0.1", function () {
      console.log(
        "communicateSendIt port=" + communicatePort + '" sending ' + buf
      );
      client.write(buf);
    });
    client.on("data", function (data) {
      console.log("communicateSendIt received: " + data);
      if (data != "") myDataRoutine(data);
    });
    client.on("close", function (data) {
      console.log("communicateSendIt connection closed");
      client = null;
    });
    client.on("error", function (err) {
      console.log("communicateSendIt error: " + err); // if planpoint quit -> error: Error: connect ECONNREFUSED
    });
  }
}

function commListen(rtnData) {
  var parent = "";
  var action = "";
  var error = "";

  console.log("    commListen gets " + rtnData);

  var xmlData = makeXML(rtnData.toString());
  if (xmlData == null) {
    console.log("    commListen failed to make XML from " + rtnData);
    return false;
  }

  try {
    action = xmlData.find("response").attr("action");
  } catch (e) {
    console.log("    commListen no action attrib " + e);
  }

  if (action == undefined) {
    debugger;
  }

  console.log(" action " + action);
  console.log("	ix_currentOpCode " + ix_currentOpCode);

  if (action == "save" || action == "saveclose") {
    console.log("   commListen saved msg");
    try {
      if (ix_currentOpCode == "SaveAndExportCustomPP") {
        console.log("SaveAndExportCustomPP op");
        // <page parent="xxx1"/>
        // <page parent="xxx2"/>

        error = xmlData.find("response").attr("error");
        $(xmlData)
          .find("page")
          .each(function () {
            console.log($(this).attr("parent"));
            if (parent == "") parent = $(this).attr("parent");
            else parent += delim + $(this).attr("parent");
          });
        //parent = xmlData.find("page").attr("parent");
      } else {
        if (ix_currentOpCode == "SaveClose") {
          parent = xmlData.find("page").attr("parent");
          error = xmlData.find("response").attr("error");
        } else {
          parent = xmlData.find("response").attr("parent");
          error = xmlData.find("response").attr("error");
        }

        //var an_xmlString = (new XMLSerializer()).serializeToString(xmlData[0]);
        //console.log (" inner parent "+parent);
        //console.log (" inner error  "+error);
        //console.log ('an_xmlString = '+an_xmlString);
      }
    } catch (e) {
      console.log("    commListen on " + action + " " + e);
    }

    //if (parent==undefined) parent="";
    //if (error ==undefined) error ="";

    console.log(" parent " + parent);

    if (
      (error == "" || error == "0" || error == undefined) &&
      parent != "" &&
      parent != undefined
    ) {
      if (
        ix_currentOpCode == "SaveAndExport" ||
        ix_currentOpCode == "SaveAndExportCustom" ||
        ix_currentOpCode == "SaveAndClose"
      ) {
        doScript("ix.closeThisDoc", closedTheDoc, parent); // AdPoint type closes
      } else if (ix_currentOpCode == "SaveAndExportCustomPP") {
        doScript("ix.closeThisPlanPointDoc", closedThePlanPointDoc, parent); // PlanPoint type closes
      }
    } else {
      console.log(
        "    commListen failed to call ix.closeThisDoc() parent=" +
          parent +
          " error" +
          error
      );
    }
  } else {
    console.log("    commListen unmatched action " + action);
  }

  return;
}

function closedTheDoc(parentDocId) {
  console.log("closedTheDoc " + parentDocId);
  if (parentDocId != "") {
    var closedFileMessage =
      '<request cid="' +
      clientID +
      '" application="' +
      localStorage.ixApplicationName +
      '" action="close" parent="' +
      parentDocId +
      '"></request>\r\n\r\n';
    communicateSendIt(closedFileMessage, commListen);
  }
}

// <request cid=”xx” application=”xx” action=”close”>
//     <page parent=”com.ix.adp.model.Page.390″/>
//     <page parent=”com.ix.adp.model.Page.389″/>
//  </request>

function closedThePlanPointDoc(parentDocIdList) {
  console.log("closedTheDoc " + parentDocIdList);

  if (parentDocIdList != "") {
    var aTmp = [];
    if (parentDocIdList.indexOf(delim) >= 0) {
      aTmp = parentDocIdList.split(delim);
    } else {
      aTmp.push(parentDocIdList);
    }

    var requestXmlString =
      '<request cid="' +
      clientID +
      '" application="' +
      localStorage.ixApplicationName +
      '" action="close" />';

    var xmlData = makeXML(requestXmlString.toString());
    if (xmlData == null) {
      console.log(
        "    closedThePlanPointDoc failed to make XML from " + requestXmlString
      );
      return false;
    }

    for (var i = 0; i < aTmp.length; i++) {
      xmlData.find("request").append('<page parent="' + aTmp[i] + '" />');
    }

    //  was like this -> var closedFileMessage = '<request cid="'+clientID+'" application="'+localStorage.ixApplicationName+'" action="close" parent="'+parentDocId+'"></request>\r\n\r\n';

    var xmlString = new XMLSerializer().serializeToString(xmlData[0]);

    var closedFileMessage = xmlString + "\r\n\r\n";

    console.log(
      "closedThePlanPointDoc data to send to PlanPoint: " + xmlString
    );

    communicateSendIt(closedFileMessage, commListen);
  }
}

function communicateTellADOpen(parentStr, nameStr, viewOnly) {
  console.log(
    "communicateTellADOpen " + parentStr + " " + nameStr + " " + viewOnly
  );
  var requestXmlString = "";
  if (viewOnly == "true") {
    requestXmlString =
      '<request cid="' +
      clientID +
      '" application="' +
      localStorage.ixApplicationName +
      '" action="open" name="' +
      nameStr +
      '" viewonly="true" parent="' +
      parentStr +
      '" />' +
      "\r\n\r\n";
  } else {
    requestXmlString =
      '<request cid="' +
      clientID +
      '" application="' +
      localStorage.ixApplicationName +
      '" action="open" name="' +
      nameStr +
      '" viewonly="false" parent="' +
      parentStr +
      '" />' +
      "\r\n\r\n";
  }
  communicateSendIt(requestXmlString, commListen);
}

function communicateTellPPOpen(openedPageList) {
  console.log("communicateTellPPOpen " + openedPageList);
  if (openedPageList == "") return;
  var aTmp = [];
  var aTmp = openedPageList.split(delim);

  var requestXmlString =
    '<request cid="' +
    clientID +
    '" application="' +
    localStorage.ixApplicationName +
    '" action="open" />';

  var xmlData = makeXML(requestXmlString.toString());
  if (xmlData == null) {
    console.log(
      "    communicateTellPPOpen failed to make XML from " + requestXmlString
    );
    return false;
  }

  for (var i = 0; i < aTmp.length; i++) {
    xmlData
      .find("request")
      .append('<page parent="' + stripDownToNumber(aTmp[i]) + '" />');
  }
  var xmlString = new XMLSerializer().serializeToString(xmlData[0]);

  var openedFileMessage = xmlString + "\r\n\r\n";

  console.log("communicateTellPPOpen data to send to PlanPoint: " + xmlString);

  communicateSendIt(openedFileMessage, commListen);
}

function stripDownToNumber(str) {
  if (str.indexOf(".") < 0) return str;
  var aTmp = [];
  aTmp = str.split(".");

  return aTmp[aTmp.length - 1];
}
